@@ -44,8 +44,8 @@ class DiscreteChannel(Channel):
             # DiscreteChannel(remote=remote, local=local, blocking=blocking, reuse=reuse),
             self.__blocking = blocking
             self.__reuse = reuse
-            self._sock = None
             # setup inner socket
+            self._sock = None
             sock = self.__setup()
             # bind to local address
             if local is not None:
@@ -117,16 +117,18 @@ class DiscreteChannel(Channel):
             # assert isinstance(sock, socket.socket)
             return sock.getpeername()
 
-    def bind(self, host: str, port: int):
+    def bind(self, address: Optional[tuple] = None, host: Optional[str] = '0.0.0.0', port: Optional[int] = 0):
+        if address is None:
+            address = (host, port)
         sock = self.__setup()
         # assert isinstance(sock, socket.socket)
-        address = (host, port)
         sock.bind(address)
 
-    def connect(self, host: str, port: int):
+    def connect(self, address: Optional[tuple] = None, host: Optional[str] = '127.0.0.1', port: Optional[int] = 0):
+        if address is None:
+            address = (host, port)
         sock = self.__setup()
         # assert isinstance(sock, socket.socket)
-        address = (host, port)
         sock.connect(address)
 
     def disconnect(self):
@@ -174,7 +176,26 @@ class DiscreteChannel(Channel):
         sock = self._sock
         if sock is None:
             raise socket.error('socket lost, cannot receive data')
-        data, remote = sock.recvfrom(max_len)
+        try:
+            remote = sock.getpeername()
+        except socket.error as error:
+            if error.errno == 57 and error.strerror == 'Socket is not connected':
+                remote = None
+            else:
+                raise error
+        try:
+            if remote is None:
+                # UDP receiving
+                data, remote = sock.recvfrom(max_len)
+            else:
+                # connected?
+                data = sock.recv(max_len)
+        except socket.error as error:
+            if not self.__blocking:
+                if error.errno == 35 and error.strerror == 'Resource temporarily unavailable':
+                    # received nothing
+                    return None, None
+            raise error
         if data is None or len(data) == 0:
             if sock.gettimeout() is None:
                 raise socket.error('remote peer reset socket')
@@ -184,5 +205,16 @@ class DiscreteChannel(Channel):
         sock = self._sock
         if sock is None:
             raise socket.error('socket lost, cannot send data: %d byte(s)' % len(data))
-        # sock.sendall(data)
+        try:
+            remote = sock.getpeername()
+        except socket.error as error:
+            if error.errno == 57 and error.strerror == 'Socket is not connected':
+                remote = None
+            else:
+                raise error
+        if remote is not None:
+            # connected?
+            assert remote == target, 'target address error: %s, %s' % (target, remote)
+            return self.write(data=data)
+        # UDP sending
         return sock.sendto(data, target)
