@@ -30,14 +30,15 @@
 
 import weakref
 from abc import ABC, abstractmethod
+from typing import Optional
 
 from tcp import Channel, Connection, ConnectionDelegate
+from tcp import BaseHub
 
 from .net import PackageConnection, ActivePackageConnection
-from .net import BasePackageHub
 
 
-class PackageHub(BasePackageHub):
+class PackageHub(BaseHub):
     """ Base Package Hub """
 
     def __init__(self, delegate: ConnectionDelegate):
@@ -48,38 +49,54 @@ class PackageHub(BasePackageHub):
     def delegate(self) -> ConnectionDelegate:
         return self.__delegate()
 
-    def create_connection(self, remote: tuple, local: tuple) -> Connection:
+    def create_connection(self, remote: Optional[tuple], local: Optional[tuple]) -> Connection:
+        # create connection with channel
         sock = self.create_channel(remote=remote, local=local)
-        conn = PackageConnection(channel=sock)
+        conn = PackageConnection(remote=remote, local=local, channel=sock)
+        # set delegate
         if conn.delegate is None:
             conn.delegate = self.delegate
-        conn.start()  # start FSM
+        # start FSM
+        conn.start()
         return conn
 
     @abstractmethod
-    def create_channel(self, remote: tuple, local: tuple) -> Channel:
+    def create_channel(self, remote: Optional[tuple], local: Optional[tuple]) -> Channel:
         raise NotImplemented
 
+    def send_command(self, body: bytes, source: Optional[tuple], destination: tuple):
+        conn = self.connect(remote=destination, local=source)
+        if isinstance(conn, PackageConnection):
+            conn.send_command(body=body, source=source, destination=destination)
+
+    def send_message(self, body: bytes, source: Optional[tuple], destination: tuple):
+        conn = self.connect(remote=destination, local=source)
+        if isinstance(conn, PackageConnection):
+            conn.send_message(body=body, source=source, destination=destination)
+
 
 class ActivePackageHub(PackageHub, ABC):
     """ Active Package Hub """
 
-    def create_connection(self, remote: tuple, local: tuple) -> Connection:
+    def create_connection(self, remote: Optional[tuple], local: Optional[tuple]) -> Connection:
+        # create connection with addresses
         conn = ActiveDiscreteConnection(remote=remote, local=local, hub=self)
+        # set delegate
         if conn.delegate is None:
             conn.delegate = self.delegate
-        conn.start()  # start FSM
+        # start FSM
+        conn.start()
         return conn
 
 
 class ActiveDiscreteConnection(ActivePackageConnection):
     """ Active Discrete Package Connection """
 
-    def __init__(self, remote: tuple, local: tuple, hub: ActivePackageHub):
+    def __init__(self, remote: tuple, local: Optional[tuple], hub: ActivePackageHub):
         super().__init__(remote=remote, local=local)
         self.__hub = weakref.ref(hub)
 
-    def connect(self, remote: tuple, local: tuple) -> Channel:
+    def connect(self, remote: tuple, local: Optional[tuple] = None) -> Channel:
         hub = self.__hub()
-        # assert isinstance(hub, ActiveStreamHub)
+        # assert isinstance(hub, ActivePackageHub)
         return hub.create_channel(remote=remote, local=local)
